/*
     This file is part of GNUnet.
     (C) 2001, 2002 Christian Grothoff (and other contributing authors)

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/

/**
 * Main function to download files on GNUnet.
 *
 * @author Christian Grothoff
 * @file applications/afs/tools/gnunet-download.c 
 **/

#include "config.h"
#include <unistd.h>
#include "applications/afs/afs.h"
#include "applications/afs/encoding/downloadutil.h"
#include "applications/afs/util/policy.h"
#include "applications/afs/encoding/requestmanager.h"
#include "applications/common/printhelp.h"
#include "util/returnvalues.h"
#include "util/logging.h"
#include "util/getopt.h"
#include "util/cron.h"
#include "util/tcpio.h"
#include "util/configuration.h"
#include "util/initialize.h"

/**
 * Prints the usage information for this command if the user errs.
 * Aborts the program.
 **/
static void printhelp() {
  printf("USAGE: gnunet-download [OPTIONS] "\
	 "HASH1 HASH2 CRC32 SIZE\n"\
	 "-a LEVEL, --anonymity=LEVEL   : "\
	 "set receiver-anonymity enforcing to LEVEL\n"\
	 "-o FILENAME, --output=FILENAME: "\
	 "write the file to FILENAME (required)\n"\
	 "-V, --verbose                 : "\
	 "print download progression\n");
  printHelpDefault();
}

/**
 * ParseOptions for gnunet-download.
 * @return SYSERR to abort afterwards, OK to continue
 **/
static int parseOptions(int argc,
			char ** argv) {
  int c;  
  int crc32;
  size_t filesize;

  freeNonNull(setConfigurationString("GNUNETD",
				     "LOGFILE",
				     NULL),
	      "gnunet-download always use option d");
  while (1) {
    int option_index = 0;
    static struct option long_options[] = {
      LONG_DEFAULT_OPTIONS,
      { "anonymity", 1, 0, 'a' }, 
      { "verbose",   0, 0, 'V' },
      { "output",    1, 0, 'o' },
      { 0,0,0,0 }
    };    
    c = getopt_long(argc,
		    argv, 
		    "a:vhdc:L:H:Vo:", 
		    long_options, 
		    &option_index);    
    if (c == -1) 
      break;  // No more flags to process
    if (YES == parseDefaultOptions(c, optarg))
      continue;
    switch(c) {
    case 'a': {
      unsigned int receivePolicy;
      
      if (1 != sscanf(optarg, "%ud", &receivePolicy)) {
        LOG(LOG_FAILURE,
	    "FAILURE: You must pass a number to the -a option.\n");
	return -1;
      }
      setConfigurationInt("AFS",
      			  "ANONYMITY-RECEIVE",
			  receivePolicy);
      break;
    }
    case 'o':
      freeNonNull(setConfigurationString("GNUNET-DOWNLOAD",
					 "FILENAME",
					 optarg),
		  "gnunet-download option parsing o");
      break;
    case 'v': 
      printf("GNUnet v%s, gnunet-download v%s\n",
	     VERSION,
	     AFS_VERSION);
      return SYSERR;
    case 'V':
      freeNonNull(setConfigurationString("GNUNET-DOWNLOAD",
					 "VERBOSE",
					 "YES"),
		  "gnunet-download option parsing V");
      break;
    case 'h': 
      printhelp(); 
      return SYSERR;
    default: 
      LOG(LOG_FAILURE,
	  "Unknown option %c. Aborting.\n"\
	  "Use --help to get a list of options.\n",
	  c);
      return SYSERR;
    } /* end of parsing commandline */
  } /* while (1) */
  if (argc - optind != 4) {
    LOG(LOG_WARNING, 
	"Not enough arguments. "\
	"You must specify a key-hash, query-hash, "\
	"filesize and crc.\n");
    printhelp();
    return SYSERR;
  } 
  freeNonNull(setConfigurationString("GNUNET-DOWNLOAD",
				     "KEY-HASH",
				     argv[optind++]),
	      "parsing keyhash option");
  freeNonNull(setConfigurationString("GNUNET-DOWNLOAD",
				     "QUERY-HASH",
				     argv[optind++]),
	      "parsing queryhash option");
  if (1 != sscanf(argv[optind++],
		  "%X",
		  &crc32)) 
    errexit(">>%s<< is invalid for the CRC code (must be a hexadecimal number)\n",
	    argv[--optind]);
  setConfigurationInt("GNUNET-DOWNLOAD",
		      "ROOT-CRC32",
		      crc32);
  if (1 != sscanf(argv[optind++],
		  "%u",
		  &filesize))
    errexit(">>%s<< is invalid for the filesize (must be a positive number)\n",
	    argv[--optind]);
  setConfigurationInt("GNUNET-DOWNLOAD",
		      "FILESIZE",
		      filesize);
  return OK;
}

/**
 * This method is called whenever data is received.
 * The current incarnation just ensures that the main
 * method exits once the download is complete.
 **/
static void progressModel(ProgressStats * stats,
			  void * data) {
  static int prevfilesize=-1;

  Semaphore * sem = data;
  if (YES == testConfigurationString("GNUNET-DOWNLOAD",
				     "VERBOSE",
				     "YES")) {
    if(stats->progress != prevfilesize)
      printf("Download at %8u out of %8u bytes\n",
  	     stats->progress, 
	     stats->filesize);
    prevfilesize = stats->progress;
  }
  if (stats->progress == stats->filesize)
    semaphore_up(sem);
}

/**
 * the main function to insert files into GNUnet.
 * @param argc number of arguments from the command line
 * @param argv command line arguments
 * @return return value from insertFile: 0: ok, -1: file not found 
 **/   
int main(int argc, char ** argv) {
  CHK_Hashes topCHK;
  int crc32;
  size_t filesize;
  char * fileName;
  HexName * hashStr;
  Semaphore * doneSem;
  RequestManager * rm;
  cron_t startTime;

  if (SYSERR == initUtil(argc, argv, &parseOptions))
    return 0;

  cronTime(&startTime);
  fileName = getConfigurationString("GNUNET-DOWNLOAD",
				    "FILENAME");
  if (fileName == NULL) {
    LOG(LOG_ERROR,
	"ERROR: You must specify a filename (option -o).\n");
    printhelp();
    return -1;
  }  
  crc32 = getConfigurationInt("GNUNET-DOWNLOAD",
			      "ROOT-CRC32");
  filesize = getConfigurationInt("GNUNET-DOWNLOAD",
				 "FILESIZE");
  hashStr = (HexName*) getConfigurationString("GNUNET-DOWNLOAD",
					      "KEY-HASH");
  hex2hash(hashStr,
	   &topCHK.key);
  xfree(hashStr, "freeing hash string");
  hashStr = (HexName*) getConfigurationString("GNUNET-DOWNLOAD",
					      "QUERY-HASH");
  hex2hash(hashStr,
	   &topCHK.query);
  xfree(hashStr, "freeing hash string");

  startCron();
  initAnonymityPolicy(NULL);
  doneSem = new_semaphore(0);
  /* insert request for root-node here! */
  rm = downloadFile(&topCHK,
		    crc32, 
		    filesize,
		    fileName, 
		    &progressModel,
		    doneSem);
  /* wait here until callback indicates that the file
     is complete: */
  semaphore_down(doneSem);
  LOG(LOG_DEBUG,
      "DEBUG: download complete\n");
  printf("Speed was %8.3f kilobyte per second.\n",
	 (filesize/1024.0) / 
	 (((double)(cronTime(NULL)-startTime)) / (double)cronSECONDS) );

  free_semaphore(doneSem);
  doneAnonymityPolicy();
  destroyRequestManager(rm);
  stopCron();
  doneUtil();

  return 0;
}

/* end of gnunet-download.c */
