/*
     This file is part of GNUnet

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/
/** 
 * @file include/util/statuscalls.h
 * @author Tzvetan Horozov
 * @author Christian Grothoff
 *
 * Status calls implementation.  Usage is how much we are using
 * (relative to what is available). Load is what we are using relative
 * to what we are allowed to use.
 **/

#ifndef STATUSCALLS_H
#define STATUSCALLS_H

#include "config.h"
#include "util/storage.h"

/**
 * definition of a callback method for drive-full notifications 
 * @param spaceLeft increment the value referenced here by the
 *   amount of space reclaimed. Will initially always be negative.
 **/
typedef void (*DriveFullCallback)(long long * spaceLeft);


/**
 * The following routine returns the percentage of available used
 * bandwidth. A number from 0-100 is returned.  Example: If 81 is
 * returned this means that 81% of the network bandwidth of the host
 * is consumed.
 */
int networkUsage();

/**
 * The following routine returns a number between 0-100 which 
 * indicates the percentage of disk space used
 */
int diskUsage();

/**
 * The following routine returns a number between 0-100 which indicates
 * the percentage CPU usage.
 */
int cpuUsage();

/**
 * Get the load of the CPU relative to what is allowed.
 * @return the CPU load as a percentage of allowed
 *        (100 is equivalent to full load)
 **/
int getCPULoad();

/**
 * Get the load of the network relative to what is allowed.
 * @return the network load as a percentage of allowed
 *        (100 is equivalent to full load)
 **/
int getNetworkLoad();

/**
 * Tell statuscalls to increment the number of bytes sent
 **/
void incrementBytesSent(unsigned long long delta);

/**
 * Tell statuscalls to increment the number of bytes received
 **/
void incrementBytesReceived(unsigned long long delta);

/**
 * We have used approximately kb KB of storage space more.
 **/
void increaseKilobyteUsed(int kb);
	
/**
 * The following method is called in order to initialize the status
 * calls routines. After that it is safe to call each of the status
 * calls separately
 */
void initStatusCalls();

/**
 * Re-read the configuration for statuscalls.
 **/
void resetStatusCalls();

/**
 * Shutdown the module.
 **/
void doneStatusCalls();

/**
 * Return the number of kilobytes GNUnet is allowed to use.
 **/
unsigned long long getDiskQuota();

/**
 * Return the number of kilobytes we have left.
 **/
long long getSpaceLeft();

/**
 * Register a file for monitoring space consumption.
 **/
int registerSpaceMonitor(char * filename);

/**
 * Register a callback for drive-full monitoring
 **/
int registerDriveFullCallback(DriveFullCallback callback);

/**
 * Unregister a callback for drive-full monitoring.
 **/
int unregisterDriveFullCallback(DriveFullCallback callback);

#endif
